@@ -46,7 +46,7 @@ public class SweetHome3DBootstrap {
         "iText-2.1.7.jar", // Jars included in Sweet Home 3D executable jar file 
         "freehep-vectorgraphics-svg-2.1.1b.jar",
         "Loader3DS1_2u.jar",
-        "sunflow-0.07.3h.jar",
+        "sunflow-0.07.3i.jar",
         "jmf.jar",
         "batik-svgpathparser-1.7.jar",
         "jnlp.jar"}));
@@ -120,8 +120,8 @@ public class SweetHome3DBootstrap {
     } else {
       extensionJarsAndDlls.addAll(Arrays.asList(new String [] {
           "linux/i386/libj3dcore-ogl.so", // Linux 32 bits DLLs
-          "linux/i386/libj3dcore-ogl-cg.so", // Windows 32 bits DLLs
-          "windows/i386/j3dcore-d3d.dll",
+          "linux/i386/libj3dcore-ogl-cg.so", 
+          "windows/i386/j3dcore-d3d.dll", // Windows 32 bits DLLs
           "windows/i386/j3dcore-ogl.dll",
           "windows/i386/j3dcore-ogl-cg.dll",
           "windows/i386/j3dcore-ogl-chk.dll"}));
@@ -154,12 +154,12 @@ public class SweetHome3DBootstrap {
             // be deleted if they are still opened when program exits 
             new File(System.getProperty("java.io.tmpdir")), applicationClassName + "-cache-")  
         : new ExtensionsClassLoader(
-              sweetHome3DBootstrapClass.getClassLoader(), 
-              sweetHome3DBootstrapClass.getProtectionDomain(),
-              extensionJarsAndDlls.toArray(new String [extensionJarsAndDlls.size()]), applicationPackages);      
+            sweetHome3DBootstrapClass.getClassLoader(), 
+            sweetHome3DBootstrapClass.getProtectionDomain(),
+            extensionJarsAndDlls.toArray(new String [extensionJarsAndDlls.size()]), applicationPackages);      
     Class applicationClass = java3DClassLoader.loadClass(applicationClassName);
     Method applicationClassMain = 
-      applicationClass.getMethod("main", Array.newInstance(String.class, 0).getClass());
+        applicationClass.getMethod("main", Array.newInstance(String.class, 0).getClass());
     // Call application class main method with reflection
     applicationClassMain.invoke(null, new Object [] {args});
   }
