@@ -26,11 +26,12 @@ import java.io.InputStream;
 import java.io.InterruptedIOException;
 import java.io.ObjectOutputStream;
 import java.io.OutputStream;
+import java.io.OutputStreamWriter;
 import java.net.URISyntaxException;
 import java.net.URL;
 import java.net.URLEncoder;
 import java.util.ArrayList;
-import java.util.HashMap;
+import java.util.LinkedHashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.zip.ZipEntry;
@@ -49,10 +50,8 @@ import com.eteks.sweethome3d.tools.URLContent;
  * @see DefaultHomeInputStream
  */
 public class DefaultHomeOutputStream extends FilterOutputStream {
-  private int                    compressionLevel;
-  private ContentRecording       contentRecording;
-  private List<Content>          contents           = new ArrayList<Content>();
-  private Map<URL, List<String>> zipUrlEntriesCache = new HashMap<URL, List<String>>();
+  private int              compressionLevel;
+  private ContentRecording contentRecording;
   
   /**
    * Creates a stream that will serialize a home and all the contents it references
@@ -113,17 +112,31 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
     ZipOutputStream zipOut = new ZipOutputStream(this.out);
     zipOut.setLevel(this.compressionLevel);
     checkCurrentThreadIsntInterrupted();
-    // Write home in first entry in a file "Home"
+    // Write home in the first entry named "Home"
     zipOut.putNextEntry(new ZipEntry("Home"));
     // Use an ObjectOutputStream that keeps track of Content objects
-    ObjectOutputStream objectOut = new HomeObjectOutputStream(zipOut);
+    HomeObjectOutputStream objectOut = new HomeObjectOutputStream(zipOut);
     objectOut.writeObject(home);
     objectOut.flush();
     zipOut.closeEntry();
-    // Write Content objects in files "0" to "n"
-    for (int i = 0, n = contents.size(); i < n; i++) {
-      Content content = contents.get(i);
-      String entryNameOrDirectory = String.valueOf(i);
+    List<Content> savedContents = objectOut.getSavedContents();
+    if (savedContents.size() > 0) {
+      // In the second entry named "ContentDigests", write content digests to help repair damaged files     
+      zipOut.putNextEntry(new ZipEntry("ContentDigests"));
+      OutputStreamWriter writer = new OutputStreamWriter(zipOut, "UTF-8");
+      ContentDigestManager digestManager = ContentDigestManager.getInstance();
+      writer.write("ContentDigests-Version: 1.0\n\n");
+      for (Content content : savedContents) {
+        writer.write("Name: " + objectOut.getContentEntry(content) + "\n");
+        writer.write("SHA-1-Digest: " + Base64.encodeBytes(digestManager.getContentDigest(content)) + "\n\n");
+      }
+      writer.flush();
+      zipOut.closeEntry();
+    }
+    // Write Content objects in files "0" to "n - 1"
+    int i = 0;
+    for (Content content : savedContents) {
+      String entryNameOrDirectory = String.valueOf(i++);
       if (content instanceof ResourceURLContent) {
         writeResourceZipEntries(zipOut, entryNameOrDirectory, (ResourceURLContent)content);
       } else if (content instanceof URLContent
@@ -159,7 +172,7 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
           // Consider content is a multi part resource only if it's in a subdirectory
           String entryDirectory = entryName.substring(0, lastSlashIndex + 1);
           // Write in home stream each zipped stream entry that is stored in the same directory  
-          for (String zipEntryName : getZipUrlEntries(zipUrl)) {
+          for (String zipEntryName : ContentDigestManager.getInstance().getZipURLEntries(urlContent)) {
             if (zipEntryName.startsWith(entryDirectory)) {
               Content siblingContent = new URLContent(new URL("jar:" + zipUrl + "!/" 
                   + URLEncoder.encode(zipEntryName, "UTF-8").replace("+", "%20")));
@@ -194,30 +207,6 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
     }
   }
 
-  /**
-   * Returns the list of entries contained in <code>zipUrl</code>.
-   */
-  private List<String> getZipUrlEntries(URL zipUrl) throws IOException {
-    List<String> zipUrlEntries = this.zipUrlEntriesCache.get(zipUrl);
-    if (zipUrlEntries == null) {
-      zipUrlEntries = new ArrayList<String>();
-      this.zipUrlEntriesCache.put(zipUrl, zipUrlEntries);
-      ZipInputStream zipIn = null;
-      try {
-        // Search all entries of zip url
-        zipIn = new ZipInputStream(zipUrl.openStream());
-        for (ZipEntry entry; (entry = zipIn.getNextEntry()) != null; ) {
-          zipUrlEntries.add(entry.getName());
-        }
-      } finally {
-        if (zipIn != null) {
-          zipIn.close();
-        }
-      }
-    }
-    return zipUrlEntries;
-  }
-  
   /**
    * Writes in <code>zipOut</code> stream one or more entries matching the content
    * <code>urlContent</code> coming from a home file.
@@ -232,7 +221,7 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
       URL zipUrl = urlContent.getJAREntryURL();
       String entryDirectory = entryName.substring(0, slashIndex + 1);
       // Write in home stream each zipped stream entry that is stored in the same directory  
-      for (String zipEntryName : getZipUrlEntries(zipUrl)) {
+      for (String zipEntryName : ContentDigestManager.getInstance().getZipURLEntries(urlContent)) {
         if (zipEntryName.startsWith(entryDirectory)) {
           Content siblingContent = new URLContent(new URL("jar:" + zipUrl + "!/" 
               + URLEncoder.encode(zipEntryName, "UTF-8").replace("+", "%20")));
@@ -297,6 +286,8 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
    * by temporary <code>URLContent</code> objects and stores them in a list.
    */
   private class HomeObjectOutputStream extends ObjectOutputStream {
+    private Map<Content, URLContent> savedContents = new LinkedHashMap<Content, URLContent>();
+
     public HomeObjectOutputStream(OutputStream out) throws IOException {
       super(out);
       if (contentRecording != ContentRecording.INCLUDE_NO_CONTENT) {
@@ -309,12 +300,17 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
       if (obj instanceof TemporaryURLContent 
           || obj instanceof HomeURLContent
           || (contentRecording == ContentRecording.INCLUDE_ALL_CONTENT && obj instanceof Content)) {
-        // Add obj to Content objects list
-        contents.add((Content)obj);
-
         String subEntryName = "";
         if (obj instanceof URLContent) {
           URLContent urlContent = (URLContent)obj;
+          // Check if duplicated content can be avoided 
+          ContentDigestManager contentDigestManager = ContentDigestManager.getInstance();
+          for (Map.Entry<Content, URLContent> contentEntry : savedContents.entrySet()) {
+            if (contentDigestManager.equals(urlContent, contentEntry.getKey())) {
+              return contentEntry.getValue();
+            }
+          }
+          checkCurrentThreadIsntInterrupted();
           // If content comes from a zipped content  
           if (urlContent.isJAREntry()) {
             String entryName = urlContent.getJAREntryName();
@@ -356,10 +352,27 @@ public class DefaultHomeOutputStream extends FilterOutputStream {
         } 
 
         // Return a temporary URL that points to content object 
-        return new URLContent(new URL("jar:file:temp!/" + (contents.size() - 1) + subEntryName));
+        URLContent tmpContent = new URLContent(new URL("jar:file:temp!/" + this.savedContents.size() + subEntryName));
+        // Add obj to Content objects list
+        this.savedContents.put((Content)obj, tmpContent);
+        return tmpContent;
       } else {
         return obj;
       }
     }
+    
+    /**
+     * Returns the contents to be saved.
+     */
+    public List<Content> getSavedContents() {
+      return new ArrayList<Content>(this.savedContents.keySet());
+    }
+
+    /**
+     * Returns the entry of a given <code>content</code>.
+     */
+    public String getContentEntry(Content content) {
+      return this.savedContents.get(content).getJAREntryName();
+    }
   }
 }
\ No newline at end of file
