@@ -29,14 +29,17 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.WeakHashMap;
 
 import javax.media.j3d.BranchGroup;
 import javax.media.j3d.ColoringAttributes;
 import javax.media.j3d.LineAttributes;
 import javax.media.j3d.Material;
 import javax.media.j3d.PolygonAttributes;
+import javax.media.j3d.Texture;
 import javax.vecmath.Color3f;
 
+import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.Room;
 
 /**
@@ -55,7 +58,8 @@ public abstract class Object3DBranch extends BranchGroup {
   protected static final Integer  DEFAULT_AMBIENT_COLOR = 0x333333;
   protected static final Material DEFAULT_MATERIAL      = new Material();
 
-  private static final Map<Long, Material> materials = new HashMap<Long, Material>();
+  private static final Map<Long, Material>              materials = new HashMap<Long, Material>();
+  private static final Map<Home, Map<Texture, Texture>> homesTextures = new WeakHashMap<Home, Map<Texture, Texture>>();
   
   static {
     DEFAULT_MATERIAL.setCapability(Material.ALLOW_COMPONENT_READ);
@@ -68,6 +72,30 @@ public abstract class Object3DBranch extends BranchGroup {
    */
   public abstract void update();
 
+  /**
+   * Returns a cloned instance of texture shared per <code>home</code> or 
+   * the texture itself if <code>home</code> is <code>null</code>.
+   * As sharing textures across universes might cause some problems, 
+   * it's safer to handle a copy of textures for a given home. 
+   */
+  protected Texture getHomeTextureClone(Texture texture, Home home) {
+    if (home == null || texture == null) {
+      return texture;
+    } else {
+      Map<Texture, Texture> homeTextures = homesTextures.get(home);
+      if (homeTextures == null) {
+        homeTextures = new WeakHashMap<Texture, Texture>();
+        homesTextures.put(home, homeTextures);
+      }
+      Texture clonedTexture = homeTextures.get(texture);
+      if (clonedTexture == null) {
+        clonedTexture = (Texture)texture.cloneNodeComponent(false);
+        homeTextures.put(texture, clonedTexture);
+      }
+      return clonedTexture;
+    }
+  }
+  
   /**
    * Returns the shape matching the coordinates in <code>points</code> array.
    */
@@ -195,35 +223,54 @@ public abstract class Object3DBranch extends BranchGroup {
         areaHoles.clear();
       }
     } else {
+      List<Area> subAreas = new ArrayList<Area>(areaPointsLists.size());
       for (List<float []> holePoints : areaHolesLists) {
-        // Search the closest points in the current area and the hole part
-        float minDistance = Float.MAX_VALUE;
-        int holeClosestPointIndex = 0;
-        int areaClosestPointIndex = 0;
-        List<float []> closestAreaPartPoints = null;
-        for (int i = 0; i < areaPointsLists.size() && minDistance > 0; i++) {
-          List<float []> areaPartPoints = areaPointsLists.get(i);
-          for (int j = 0; j < holePoints.size() && minDistance > 0; j++) {
-            for (int k = 0; k < areaPartPoints.size() && minDistance > 0; k++) {
-              float distance = (float)Point2D.distanceSq(holePoints.get(j) [0], holePoints.get(j) [1],
-                  areaPartPoints.get(k) [0], areaPartPoints.get(k) [1]);
-              if (distance < minDistance) {
-                minDistance = distance;
-                holeClosestPointIndex = j;
-                areaClosestPointIndex = k;
-                closestAreaPartPoints = areaPartPoints;
-              }
+        List<float []> enclosingAreaPartPoints = null;
+        if (areaPointsLists.size() == 1) {
+          enclosingAreaPartPoints = areaPointsLists.get(0);
+        } else {
+          // Search the sub area that contains the current hole
+          for (int i = 0; i < areaPointsLists.size() && enclosingAreaPartPoints == null; i++) {
+            Area subArea;
+            if (i >= subAreas.size()) {
+              List<float []> areaPartPoints = areaPointsLists.get(i);
+              subArea = new Area(getShape(areaPartPoints.toArray(new float [areaPartPoints.size()][])));
+              subAreas.add(subArea);
+            } else {
+              subArea = subAreas.get(i);
+            }
+            if (subArea.contains(holePoints.get(0) [0], holePoints.get(0) [1])) {
+              enclosingAreaPartPoints = areaPointsLists.get(i);
             }
           }
         }
-        // Combine the areas at their closest points
-        if (minDistance != 0) {
-          closestAreaPartPoints.add(areaClosestPointIndex, closestAreaPartPoints.get(areaClosestPointIndex));
-          closestAreaPartPoints.add(++areaClosestPointIndex, holePoints.get(holeClosestPointIndex));
+        
+        // In some very rare cases, ignore very small areas that were wrongly considered as holes
+        if (enclosingAreaPartPoints != null) {
+          // Search the closest points in the enclosing area and the hole part
+          float minDistance = Float.MAX_VALUE;
+          int holeClosestPointIndex = 0;
+          int areaClosestPointIndex = 0;
+          for (int i = 0; i < holePoints.size() && minDistance > 0; i++) {
+            for (int j = 0; j < enclosingAreaPartPoints.size() && minDistance > 0; j++) {
+              float distance = (float)Point2D.distanceSq(holePoints.get(i) [0], holePoints.get(i) [1],
+                  enclosingAreaPartPoints.get(j) [0], enclosingAreaPartPoints.get(j) [1]);
+              if (distance < minDistance) {
+                minDistance = distance;
+                holeClosestPointIndex = i;
+                areaClosestPointIndex = j;
+              }
+            }
+          }
+          // Combine the areas at their closest points
+          if (minDistance != 0) {
+            enclosingAreaPartPoints.add(areaClosestPointIndex, enclosingAreaPartPoints.get(areaClosestPointIndex));
+            enclosingAreaPartPoints.add(++areaClosestPointIndex, holePoints.get(holeClosestPointIndex));
+          }
+          List<float []> lastPartPoints = holePoints.subList(holeClosestPointIndex, holePoints.size());
+          enclosingAreaPartPoints.addAll(areaClosestPointIndex, lastPartPoints);
+          enclosingAreaPartPoints.addAll(areaClosestPointIndex + lastPartPoints.size(), holePoints.subList(0, holeClosestPointIndex));
         }
-        List<float []> lastPartPoints = holePoints.subList(holeClosestPointIndex, holePoints.size());
-        closestAreaPartPoints.addAll(areaClosestPointIndex, lastPartPoints);
-        closestAreaPartPoints.addAll(areaClosestPointIndex + lastPartPoints.size(), holePoints.subList(0, holeClosestPointIndex));
       }
       
       for (List<float []> pathPoints : areaPointsLists) {
