@@ -23,6 +23,9 @@ import java.awt.Color;
 import java.awt.EventQueue;
 import java.awt.Graphics;
 import java.awt.Graphics2D;
+import java.awt.RenderingHints;
+import java.awt.TexturePaint;
+import java.awt.geom.Rectangle2D;
 import java.awt.image.BufferedImage;
 import java.io.IOException;
 import java.io.InputStream;
@@ -42,6 +45,7 @@ import javax.media.j3d.ImageComponent2D;
 import javax.media.j3d.Texture;
 
 import com.eteks.sweethome3d.model.Content;
+import com.eteks.sweethome3d.model.HomeTexture;
 import com.eteks.sweethome3d.tools.URLContent;
 import com.sun.j3d.utils.image.TextureLoader;
 
@@ -50,26 +54,26 @@ import com.sun.j3d.utils.image.TextureLoader;
  * @author Emmanuel Puybaret
  */
 public class TextureManager {
-  private static TextureManager          instance;
+  private static TextureManager instance;
   // Image used if an image content couldn't be loaded
-  private final Texture                  errorTexture;
+  private final Texture         errorTexture;
   // Image used while an image content is loaded
-  private final Texture                  waitTexture;
-  // Map storing loaded texture contents
-  private final Map<Content, TextureKey> contentTextureKeys;
-  // Map storing loaded textures
-  private final Map<TextureKey, Texture> textures;
-  // Map storing model nodes being loaded
-  private Map<Content, List<TextureObserver>> loadingTextureObservers;
+  private final Texture         waitTexture;
+  // Map storing loaded rotated texture contents
+  private final Map<Content, List<ComparableTextureAngleTuple>> contentTextures;
+  // Map storing loaded textures used to compare their images
+  private final Map<Texture, ComparableTexture>                 textures;
+  // Map storing model textures being loaded
+  private Map<RotatedContentKey, List<TextureObserver>>         loadingTextureObservers;
   // Executor used to load images
-  private ExecutorService                texturesLoader;
+  private ExecutorService       texturesLoader;
 
   private TextureManager() {
     this.errorTexture = getColoredImageTexture(Color.RED);
     this.waitTexture = getColoredImageTexture(Color.WHITE);
-    this.contentTextureKeys = new WeakHashMap<Content, TextureKey>();
-    this.textures = new WeakHashMap<TextureKey, Texture>();
-    this.loadingTextureObservers = new HashMap<Content, List<TextureObserver>>();
+    this.contentTextures = new WeakHashMap<Content, List<ComparableTextureAngleTuple>>();
+    this.textures = new WeakHashMap<Texture, ComparableTexture>();
+    this.loadingTextureObservers = new HashMap<RotatedContentKey, List<TextureObserver>>();
   }
 
   /**
@@ -91,7 +95,7 @@ public class TextureManager {
       this.texturesLoader = null;
     }
     synchronized (this.textures) {
-      this.contentTextureKeys.clear();
+      this.contentTextures.clear();
       this.textures.clear();
     }
   }
@@ -141,19 +145,40 @@ public class TextureManager {
   public void loadTexture(final Content content,
                           boolean synchronous,
                           final TextureObserver textureObserver) {
-    Texture texture;
-    TextureKey textureKey;
+    loadTexture(content, 0, synchronous, textureObserver);
+  }
+
+  /**
+   * Reads a texture image from <code>content</code> notified to <code>textureObserver</code>. 
+   * If the texture isn't loaded in cache yet and <code>synchronous</code> is false, a one pixel 
+   * white image texture will be notified immediately to the given <code>textureObserver</code>, 
+   * then a second notification will be given in Event Dispatch Thread once the image texture is loaded. 
+   * If the texture is in cache, it will be notified immediately to the given <code>textureObserver</code>.
+   * @param content an object containing an image
+   * @param angle   the rotation angle applied to the image
+   * @param synchronous if <code>true</code>, this method will return only once image content is loaded.
+   * @param textureObserver the observer that will be notified once the texture is available
+   * @throws IllegalStateException if synchronous is <code>false</code> and the current thread isn't 
+   *    the Event Dispatch Thread.  
+   */
+  public void loadTexture(final Content content,
+                          final float   angle,
+                          boolean synchronous,
+                          final TextureObserver textureObserver) {
+    Texture texture = null;
     synchronized (this.textures) { // Use one mutex for both maps
-      textureKey = this.contentTextureKeys.get(content);
-      if (textureKey != null) {
-        texture = this.textures.get(textureKey);
-      } else {
-        texture = null;
+      List<ComparableTextureAngleTuple> contentTexturesList = this.contentTextures.get(content);
+      if (contentTexturesList != null) {
+        for (ComparableTextureAngleTuple textureAngleTuple : contentTexturesList) {
+          if (textureAngleTuple.getAngle() == angle) {
+            texture = textureAngleTuple.getTexture(); 
+          }
+        }
       }
     }
     if (texture == null) {
       if (synchronous) {
-        texture = shareTexture(loadTexture(content), content);
+        texture = shareTexture(loadTexture(content, angle), angle, content);
         // Notify loaded texture to observer
         textureObserver.textureUpdated(texture);
       } else if (!EventQueue.isDispatchThread()) {
@@ -165,7 +190,8 @@ public class TextureManager {
           this.texturesLoader = Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());
         }
         
-        List<TextureObserver> observers = this.loadingTextureObservers.get(content);
+        final RotatedContentKey contentKey = new RotatedContentKey(content, angle);
+        List<TextureObserver> observers = this.loadingTextureObservers.get(contentKey);
         if (observers != null) {
           // If observers list exists, content texture is already being loaded
           // register observer for future notification
@@ -174,16 +200,16 @@ public class TextureManager {
           // Create a list of observers that will be notified once content texture is loaded
           observers = new ArrayList<TextureObserver>();
           observers.add(textureObserver);
-          this.loadingTextureObservers.put(content, observers);
+          this.loadingTextureObservers.put(contentKey, observers);
 
           // Load the image in a different thread
           this.texturesLoader.execute(new Runnable () {
               public void run() {
-                final Texture texture = shareTexture(loadTexture(content), content);
+                final Texture texture = shareTexture(loadTexture(content, angle), angle, content);
                 EventQueue.invokeLater(new Runnable() {
                     public void run() {
                       // Notify loaded texture to observer
-                      for (TextureObserver observer : loadingTextureObservers.remove(content)) {
+                      for (TextureObserver observer : loadingTextureObservers.remove(contentKey)) {
                         observer.textureUpdated(texture);
                       }
                     }
@@ -202,30 +228,53 @@ public class TextureManager {
    * Returns a texture created from the image from <code>content</code>. 
    */
   public Texture loadTexture(final Content content) {
+    return loadTexture(content, 0);
+  }
+  
+  /**
+   * Returns a texture created from the image from <code>content</code>  
+   * and rotated of a given <code>angle</code> in radians. 
+   */
+  private Texture loadTexture(final Content content, float angle) {
     try {
       // Read the image 
       InputStream contentStream = content.openStream();
       BufferedImage image = ImageIO.read(contentStream);
+      if (angle != 0) {
+        double cos = Math.cos(angle);
+        double sin = Math.sin(angle);
+        BufferedImage rotatedImage = new BufferedImage((int)Math.round(Math.abs(image.getWidth() * cos) + Math.abs(image.getHeight() * sin)), 
+            (int)Math.round(Math.abs(image.getWidth() * sin) + Math.abs(image.getHeight() * cos)), image.getType());
+        Graphics2D g2D = (Graphics2D)rotatedImage.getGraphics();
+        g2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
+        g2D.setPaint(new TexturePaint(image, 
+                        new Rectangle2D.Float(0, 0, image.getWidth(), image.getHeight())));
+        g2D.rotate(angle);
+        float maxDimension = Math.max(rotatedImage.getWidth(), rotatedImage.getHeight());
+        g2D.fill(new Rectangle2D.Float(-maxDimension, -maxDimension, 3 * maxDimension, 3 * maxDimension));
+        g2D.dispose();
+        image = rotatedImage;
+      }
       contentStream.close();
       if (image != null) {
         Texture texture = new TextureLoader(image).getTexture();
         // Keep in user data the URL of the texture image
-        if (content instanceof URLContent) {
+        if (content instanceof URLContent && angle == 0) {
           texture.setUserData(((URLContent)content).getURL());
         }
         return texture;
       } else {
-        return errorTexture;
+        return this.errorTexture;
       }
     } catch (IOException ex) {
       // Too bad, we'll use errorTexture
-      return errorTexture;
+      return this.errorTexture;
     } catch (RuntimeException ex) {
       // Take into account exceptions of Java 3D 1.5 ImageException class
       // in such a way program can run in Java 3D 1.3.1
       if (ex.getClass().getName().equals("com.sun.j3d.utils.image.ImageException")) {
         // Images not supported by TextureLoader
-        return errorTexture;
+        return this.errorTexture;
       } else {
         throw ex;
       }
@@ -237,7 +286,7 @@ public class TextureManager {
    * if the same texture as the one in parameter is already shared.
    */
   public Texture shareTexture(Texture texture) {
-    return shareTexture(texture, null);
+    return shareTexture(texture, 0, null);
   }
   
   /**
@@ -246,26 +295,31 @@ public class TextureManager {
    * same texture as the one in parameter is already shared.
    */
   private Texture shareTexture(final Texture texture,
+                               final float   angle,
                                final Content content) {
-    TextureKey textureKey = new TextureKey(texture);
-    Texture sharedTexture;
+    ComparableTexture textureData = new ComparableTexture(texture);
+    Texture sharedTexture = null;
     synchronized (this.textures) { // Use one mutex for both maps
-      sharedTexture = this.textures.get(textureKey);
+      // Search which existing key matches texture key to share unique texture
+      for (Map.Entry<Texture, ComparableTexture> entry : this.textures.entrySet()) {
+        if (textureData.equalsImage(entry.getValue())) {
+          sharedTexture = entry.getKey();
+          textureData = entry.getValue(); 
+          break;
+        }
+      }
       if (sharedTexture == null) {
         sharedTexture = texture;
         setSharedTextureAttributesAndCapabilities(sharedTexture);
-        this.textures.put(textureKey, sharedTexture);
-      } else {
-        // Search which key matches sharedTexture to keep unique keys
-        for (TextureKey key : this.textures.keySet()) {
-          if (key.getTexture() == sharedTexture) {
-            textureKey = key;
-            break;
-          }
-        }
+        this.textures.put(sharedTexture, textureData);
       }
       if (content != null) {
-        this.contentTextureKeys.put(content, textureKey);
+        List<ComparableTextureAngleTuple> contentTexturesList = this.contentTextures.get(content);
+        if (contentTexturesList == null) {
+          contentTexturesList = new ArrayList<ComparableTextureAngleTuple>(1);
+          this.contentTextures.put(content, contentTexturesList);
+        }
+        contentTexturesList.add(new ComparableTextureAngleTuple(textureData, angle));
       }
     }
     return sharedTexture;
@@ -295,16 +349,40 @@ public class TextureManager {
    */
   public boolean isTextureTransparent(Texture texture) {
     synchronized (this.textures) { // Use one mutex for both maps
-      // Search which key matches texture
-      for (TextureKey key : textures.keySet()) {
-        if (key.getTexture() == texture) {
-          return key.isTransparent();
-        }
+      ComparableTexture textureData = this.textures.get(texture);
+      if (textureData != null) {
+        return textureData.isTransparent();
       }
       return texture.getFormat() == Texture.RGBA;
     }
   }
-  
+
+  /**
+   * Returns the width of the given texture once its rotation angle is applied.
+   */
+  public float getRotatedTextureWidth(HomeTexture texture) {
+    float angle = texture.getAngle();
+    if (angle != 0) {
+      return (float)Math.rint(Math.abs(texture.getWidth() * Math.cos(angle)) 
+          + Math.abs(texture.getHeight() * Math.sin(angle)));
+    } else {
+      return texture.getWidth();
+    }
+  }
+
+  /**
+   * Returns the height of the given texture once its rotation angle is applied.
+   */
+  public float getRotatedTextureHeight(HomeTexture texture) {
+    float angle = texture.getAngle();
+    if (angle != 0) {
+      return (float)Math.rint(Math.abs(texture.getWidth() * Math.sin(angle)) 
+          + Math.abs(texture.getHeight() * Math.cos(angle)));
+    } else {
+      return texture.getHeight();
+    }
+  }
+
   /**
    * An observer that receives texture loading notifications. 
    */
@@ -313,17 +391,47 @@ public class TextureManager {
   }
   
   /**
-   * Key used to ensure texture uniqueness in textures map.
+   * Key used to ensure rotated content uniqueness.
+   */
+  private static class RotatedContentKey {
+    private Content content;
+    private float   angle;
+    
+    public RotatedContentKey(Content content, float angle) {
+      this.content = content;
+      this.angle = angle;
+    }
+    
+    @Override
+    public boolean equals(Object obj) {
+      if (this == obj) {
+        return true;
+      } else if (obj instanceof RotatedContentKey) {
+        RotatedContentKey rotatedContentKey = (RotatedContentKey)obj;
+        return this.content.equals(rotatedContentKey.content)
+            && this.angle == rotatedContentKey.angle;
+      }
+      return false;
+    }
+
+    @Override
+    public int hashCode() {
+      return this.content.hashCode() 
+          + Float.floatToIntBits(this.angle);
+    }    
+  }
+
+  /**
+   * Texture used to compare textures images and ensure texture uniqueness in textures map.
    * Image bits of the texture are stored in a weak reference to avoid grabbing memory uselessly.
    */
-  private static class TextureKey {
+  private static class ComparableTexture {
     private Texture               texture;
-    private WeakReference<int []> imageBits; 
-    private int                   hashCodeCache;
-    private boolean               hashCodeSet;
-    private boolean               transparent;
+    private WeakReference<int []> imageBits;
+    private Integer               imageBitsHashCode;
+    private Boolean               transparent;
 
-    public TextureKey(Texture texture) {
+    public ComparableTexture(Texture texture) {
       this.texture = texture;      
     }
     
@@ -334,7 +442,7 @@ public class TextureManager {
     /**
      * Returns the pixels of the given <code>image</code>.
      */
-    private int [] getImagePixels() {
+    private int [] getImageBits() {
       int [] imageBits = null;
       if (this.imageBits != null) {
         imageBits = this.imageBits.get();
@@ -361,6 +469,17 @@ public class TextureManager {
       return imageBits;
     }
 
+    /**
+     * Returns an hash code for the image of the texture that allows
+     * a faster comparison and storing images bits in a weak reference.
+     */
+    private int getImageBitsHashCode() {
+      if (this.imageBitsHashCode == null) {
+        this.imageBitsHashCode = Arrays.hashCode(getImageBits());
+      }
+      return this.imageBitsHashCode;
+    }
+    
     /**
      * Returns <code>true</code> if the image contains at least a transparent pixel. 
      */
@@ -379,6 +498,9 @@ public class TextureManager {
      * Returns <code>true</code> if the image of the texture contains at least one transparent pixel.
      */
     public boolean isTransparent() {
+      if (this.transparent == null) {
+        getImageBits();
+      }
       return this.transparent;
     }
     
@@ -386,28 +508,36 @@ public class TextureManager {
      * Returns <code>true</code> if the image of this texture and 
      * the image of the object in parameter are the same. 
      */
-    @Override
-    public boolean equals(Object obj) {
-      if (this == obj) {
+    public boolean equalsImage(ComparableTexture comparableTexture) {
+      if (this == comparableTexture) {
         return true;
-      } else if (obj instanceof TextureKey) {
-        TextureKey textureKey = (TextureKey)obj;
-        if (this.texture == textureKey.texture) {
-          return true;
-        } else if (hashCode() == textureKey.hashCode()){
-          return Arrays.equals(getImagePixels(), textureKey.getImagePixels());
-        }
+      } else if (this.texture == comparableTexture.texture) {
+        return true;
+      } else if (getImageBitsHashCode() == comparableTexture.getImageBitsHashCode()){
+        return Arrays.equals(getImageBits(), comparableTexture.getImageBits());
       }
       return false;
     }
+  }
 
-    @Override
-    public int hashCode() {
-      if (!this.hashCodeSet) {
-        this.hashCodeCache = Arrays.hashCode(getImagePixels());
-        this.hashCodeSet = true;
-      }
-      return this.hashCodeCache;
+  /** 
+   * A tuple that associates a texture and one of its possible rotation angle.
+   */
+  private static class ComparableTextureAngleTuple {
+    private ComparableTexture texture;
+    private float             angle;
+
+    public ComparableTextureAngleTuple(ComparableTexture texture, float angle) {
+      this.texture = texture;
+      this.angle = angle;
+    }
+
+    public Texture getTexture() {
+      return this.texture.getTexture();
+    }
+    
+    public float getAngle() {
+      return this.angle;
     }
   }
 }
