@@ -133,9 +133,9 @@ class FurnitureCatalogListPanel extends JPanel implements View {
   private void createComponents(FurnitureCatalog catalog,
                                 final UserPreferences preferences, 
                                 final FurnitureCatalogController controller) {
-    final CatalogListModel catalogListModel = new CatalogListModel(catalog);
+    final FurnitureCatalogListModel catalogListModel = new FurnitureCatalogListModel(catalog);
     this.catalogFurnitureList = new JList(catalogListModel) {
-        private FurnitureToolTip toolTip = new FurnitureToolTip(false, preferences);
+        private CatalogItemToolTip toolTip = new CatalogItemToolTip(false, preferences);
         private boolean mousePressed;
         private boolean firstScroll;
         
@@ -170,7 +170,7 @@ class FurnitureCatalogListPanel extends JPanel implements View {
           // Return a tooltip for furniture pieces described in the list.
           int index = locationToIndex(ev.getPoint());
           if (index != -1) {
-            this.toolTip.setPieceOfFurniture((CatalogPieceOfFurniture)getModel().getElementAt(index));
+            this.toolTip.setCatalogItem((CatalogPieceOfFurniture)getModel().getElementAt(index));
             return this.toolTip.getTipText();
           } else {
             return null;
@@ -324,11 +324,7 @@ class FurnitureCatalogListPanel extends JPanel implements View {
         furnitureCatalogPanel.searchLabel.setText(preferences.getLocalizedString(
             FurnitureCatalogListPanel.class, "searchLabel.text"));
         furnitureCatalogPanel.setMnemonics(preferences);
-        // Update categories
-        List<FurnitureCategory> categories = new ArrayList<FurnitureCategory>();
-        categories.add(null);
-        categories.addAll(preferences.getFurnitureCatalog().getCategories());
-        furnitureCatalogPanel.categoryFilterComboBox.setModel(new DefaultComboBoxModel(categories.toArray()));    
+        // Categories listed in combo box are updated through collectionChanged
       }
     }
     
@@ -463,6 +459,8 @@ class FurnitureCatalogListPanel extends JPanel implements View {
     }
     // Last row
     JScrollPane listScrollPane = new JScrollPane(this.catalogFurnitureList);
+    listScrollPane.getVerticalScrollBar().addAdjustmentListener(
+        SwingTools.createAdjustmentListenerUpdatingScrollPaneViewToolTip(listScrollPane));
     listScrollPane.setPreferredSize(new Dimension(250, 250));
     listScrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
     add(listScrollPane, 
@@ -777,72 +775,85 @@ class FurnitureCatalogListPanel extends JPanel implements View {
   /**
    * List model adaptor to CatalogPieceOfFurniture instances of catalog.  
    */
-  private static class CatalogListModel extends AbstractListModel {
+  private static class FurnitureCatalogListModel extends AbstractListModel {
     private FurnitureCatalog                catalog;
     private List<CatalogPieceOfFurniture>   furniture;
     private FurnitureCategory               filterCategory;
     private String                          filterText;
     
-    public CatalogListModel(FurnitureCatalog catalog) {
+    public FurnitureCatalogListModel(FurnitureCatalog catalog) {
       this.catalog = catalog;
-      this.furniture = new ArrayList<CatalogPieceOfFurniture>();
       this.filterText = "";
-      catalog.addFurnitureListener(new CatalogFurnitureListener(this));
-      updateFurnitureList();
+      catalog.addFurnitureListener(new FurnitureCatalogListener(this));
     }
 
     public void setFilterCategory(FurnitureCategory filterCategory) {
       this.filterCategory = filterCategory;
-      updateFurnitureList();
+      resetFurnitureList();
     }
 
     public void setFilterText(String filterText) {
       this.filterText = filterText;
-      updateFurnitureList();
+      resetFurnitureList();
     }
 
     public Object getElementAt(int index) {
+      checkFurnitureList();
       return this.furniture.get(index);
     }
 
     public int getSize() {
+      checkFurnitureList();
       return this.furniture.size();
     }
     
-    private void updateFurnitureList() {
-      this.furniture.clear();
-      for (FurnitureCategory category : this.catalog.getCategories()) {
-        for (CatalogPieceOfFurniture piece : category.getFurniture()) {
-          if ((this.filterCategory == null
-                || piece.getCategory().equals(this.filterCategory))
-              && piece.matchesFilter(this.filterText)) {
-            furniture.add(piece);
+    private void resetFurnitureList() {
+      if (this.furniture != null) {
+        this.furniture = null;
+        EventQueue.invokeLater(new Runnable() {
+            public void run() {
+              fireContentsChanged(this, -1, -1);
+            }
+          });
+      }
+    }
+
+    private void checkFurnitureList() {
+      if (this.furniture == null) {
+        this.furniture = new ArrayList<CatalogPieceOfFurniture>();
+        this.furniture.clear();
+        for (FurnitureCategory category : this.catalog.getCategories()) {
+          for (CatalogPieceOfFurniture piece : category.getFurniture()) {
+            if ((this.filterCategory == null
+                  || piece.getCategory().equals(this.filterCategory))
+                && piece.matchesFilter(this.filterText)) {
+              furniture.add(piece);
+            }
           }
         }
+        Collections.sort(this.furniture);
       }
-      Collections.sort(this.furniture);
-      fireContentsChanged(this, 0, this.furniture.size() - 1);
     }
     
     /**
-     * Catalog furniture listener bound to this tree model with a weak reference to avoid
-     * strong link between catalog and this tree.  
+     * Catalog furniture listener bound to this list model with a weak reference to avoid
+     * strong link between catalog and this list.  
      */
-    private static class CatalogFurnitureListener implements CollectionListener<CatalogPieceOfFurniture> {
-      private WeakReference<CatalogListModel>  catalogListModel;
+    private static class FurnitureCatalogListener implements CollectionListener<CatalogPieceOfFurniture> {
+      private WeakReference<FurnitureCatalogListModel>  listModel;
 
-      public CatalogFurnitureListener(CatalogListModel catalogListModel) {
-        this.catalogListModel = new WeakReference<CatalogListModel>(catalogListModel);
+      public FurnitureCatalogListener(FurnitureCatalogListModel catalogListModel) {
+        this.listModel = new WeakReference<FurnitureCatalogListModel>(catalogListModel);
       }
       
       public void collectionChanged(CollectionEvent<CatalogPieceOfFurniture> ev) {
-        // If catalog tree model was garbage collected, remove this listener from catalog
-        CatalogListModel catalogTreeModel = this.catalogListModel.get();
+        // If catalog list model was garbage collected, remove this listener from catalog
+        FurnitureCatalogListModel listModel = this.listModel.get();
         FurnitureCatalog catalog = (FurnitureCatalog)ev.getSource();
-        if (catalogTreeModel == null) {
+        if (listModel == null) {
           catalog.removeFurnitureListener(this);
         } else {
-          catalogTreeModel.updateFurnitureList();
+          listModel.resetFurnitureList();
         }
       }
     }
