@@ -22,6 +22,7 @@ package com.eteks.sweethome3d.swing;
 import java.awt.Color;
 import java.awt.Component;
 import java.awt.EventQueue;
+import java.awt.Font;
 import java.awt.Graphics;
 import java.awt.Rectangle;
 import java.awt.event.MouseAdapter;
@@ -304,9 +305,12 @@ public class FurnitureTable extends JTable implements View, Printable {
               isVisibleColumn = visibilityComponent.getBounds().contains(ev.getPoint());
             } else if (columnId == HomePieceOfFurniture.SortableProperty.NAME
                        && getValueAt(row, column) instanceof HomeFurnitureGroup) {
-              Rectangle expandedStateBounds = ((TreeTableNameCellRenderer)getCellRenderer(row, column)).
-                  getExpandedStateBounds(FurnitureTable.this, row, column);
-              isGroupRow = expandedStateBounds.contains(ev.getPoint());
+              TableCellRenderer cellRenderer = getCellRenderer(row, column);
+              if (cellRenderer instanceof TreeTableNameCellRenderer) {
+                Rectangle expandedStateBounds = ((TreeTableNameCellRenderer)cellRenderer).
+                    getExpandedStateBounds(FurnitureTable.this, row, column);
+                isGroupRow = expandedStateBounds.contains(ev.getPoint());
+              }
             }
             if (isVisibleColumn) {
               controller.toggleSelectedFurnitureVisibility();
@@ -1397,10 +1401,14 @@ public class FurnitureTable extends JTable implements View, Printable {
     private DefaultTableCellRenderer defaultRenderer;
     private JTree                    tree;
     private int                      renderedRow;
+    private Font                     defaultFont;
     
     public Component getTableCellRendererComponent(JTable table, 
          Object value, boolean isSelected, boolean hasFocus, 
          int row, int column) {
+      if (this.defaultFont == null) {
+        this.defaultFont = table.getFont();
+      }
       boolean containsGroup = false;
       for (int i = 0; i < table.getRowCount(); i++) {
         if (table.getValueAt(i, 0) instanceof HomeFurnitureGroup) {
@@ -1418,7 +1426,8 @@ public class FurnitureTable extends JTable implements View, Printable {
           this.tree.clearSelection();
         }
         this.renderedRow = row;
-        return tree;
+        this.tree.setFont(this.defaultFont);
+        return this.tree;
       } else {
         // Use default table renderer if the furniture list doesn't contain any group   
         if (this.defaultRenderer == null) {
@@ -1434,7 +1443,7 @@ public class FurnitureTable extends JTable implements View, Printable {
           iconContent = piece.getIcon();
         }
         label.setIcon(IconManager.getInstance().getIcon(
-            iconContent, table.getRowHeight() - table.getRowMargin(), table)); 
+            iconContent, table.getRowHeight() - table.getRowMargin(), table));
         return label;
       }
     }
