@@ -63,6 +63,7 @@ import javax.swing.KeyStroke;
 import javax.swing.ListModel;
 import javax.swing.ListSelectionModel;
 import javax.swing.SwingUtilities;
+import javax.swing.Timer;
 import javax.swing.event.ChangeEvent;
 import javax.swing.event.ChangeListener;
 import javax.swing.event.ListDataEvent;
@@ -150,10 +151,8 @@ public class ModelMaterialsComponent extends JButton implements View {
       ModelManager.getInstance().loadModel(controller.getModel(), new ModelManager.ModelObserver() {
           public void modelUpdated(BranchGroup modelRoot) {
             final MaterialsListModel materialsListModel = (MaterialsListModel)materialsList.getModel();
-            previewComponent.setModel(controller.getModel());
-            previewComponent.setModelRotationAndSize(controller.getModelRotation(),
+            previewComponent.setModel(controller.getModel(), controller.isBackFaceShown(), controller.getModelRotation(),
                 controller.getModelWidth(), controller.getModelDepth(), controller.getModelHeight());
-            previewComponent.setBackFaceShown(controller.isBackFaceShown());
             previewComponent.setModelMaterials(materialsListModel.getMaterials());
             materialsListModel.addListDataListener(new ListDataListener() {
                 public void contentsChanged(ListDataEvent ev) {
@@ -547,8 +546,8 @@ public class ModelMaterialsComponent extends JButton implements View {
           2, 4, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
           GridBagConstraints.NONE, new Insets(15, 0, 5, 0), 0, 0));
       add(this.shininessSlider, new GridBagConstraints(
-          2, 5, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-          GridBagConstraints.NONE, new Insets(0, 0, 5, 0), -20, 0));
+          2, 5, 2, 1, 0, 0, GridBagConstraints.NORTH, 
+          GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), -20, 0));
     }
     
     public void displayView(View parent) {
@@ -575,8 +574,20 @@ public class ModelMaterialsComponent extends JButton implements View {
             dialog.removeComponentListener(this);
           }
         });
+      // Run a timer that will make blink the selected material in preview
+      final MaterialBlinker selectedMaterialBlinker = new MaterialBlinker();
+      selectedMaterialBlinker.start();
+      this.materialsList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
+          public void valueChanged(ListSelectionEvent ev) {
+            selectedMaterialBlinker.restart();
+          }
+        });
+
       dialog.setVisible(true);
       dialog.dispose();
+      
+      selectedMaterialBlinker.stop();
+      
       this.controller.getTextureController().removePropertyChangeListener(
           TextureChoiceController.Property.TEXTURE, this.textureChangeListener);
       if (Integer.valueOf(JOptionPane.OK_OPTION).equals(optionPane.getValue())) {
@@ -663,5 +674,69 @@ public class ModelMaterialsComponent extends JButton implements View {
         return this.materials;
       }
     }
+    
+    /**
+     * A timer that makes blink the selected material in preview.
+     */
+    private class MaterialBlinker extends Timer {
+      public MaterialBlinker() {
+        super(500, null);
+        addActionListener(new ActionListener() {
+            public void actionPerformed(ActionEvent ev) {
+              toggleBlinkingState();
+            }
+          });
+      }
+
+      private void toggleBlinkingState() {
+        MaterialsListModel listModel = (MaterialsListModel)materialsList.getModel();
+        HomeMaterial [] materials = listModel.getMaterials();
+        if (listModel.getSize() > 1) {
+          if (getDelay() != 1000) {
+            setDelay(1000);
+            int selectedIndex = materialsList.getSelectedIndex();
+            if (selectedIndex != -1) {
+              if (materials == null) {
+                materials = new HomeMaterial [listModel.getSize()];
+              } else {
+                materials = materials.clone();
+              }
+              HomeMaterial defaultMaterial = listModel.getDefaultMaterialAt(selectedIndex);
+              HomeMaterial selectedMaterial = materials [selectedIndex] != null 
+                  ? materials [selectedIndex]
+                  : defaultMaterial;
+              int blinkColor = materialsList.getSelectionBackground().darker().getRGB();
+              if (selectedMaterial.getTexture() == null) {
+                Integer selectedColor = selectedMaterial.getColor();
+                if (selectedColor == null) {
+                  selectedColor = defaultMaterial.getColor();
+                }
+                int componentAverage = (((selectedColor >> 16) & 0xFF) + ((selectedColor >> 8) & 0xFF) + (selectedColor & 0xFF)) / 3;
+                if (componentAverage > 0x77) {
+                  // Display a darker color for a bright color
+                  blinkColor = new Color(selectedColor).darker().darker().getRGB();
+                } else if (componentAverage > 0x0F) {
+                  // Display a brighter color for a dark color
+                  blinkColor = new Color(selectedColor).brighter().brighter().getRGB();
+                }
+              }
+              materials [selectedIndex] = 
+                  new HomeMaterial(selectedMaterial.getName(), blinkColor, null, selectedMaterial.getShininess());
+              previewComponent.setModelMaterials(materials);
+            }
+          } else {
+            setDelay(100);
+            previewComponent.setModelMaterials(materials);
+          }
+        }
+      }
+      
+      @Override
+      public void restart() {
+        setInitialDelay(100);
+        setDelay(100);
+        super.restart();
+      }
+    }
   }
 }
\ No newline at end of file
