@@ -285,6 +285,14 @@ public class HomeController3D implements Controller {
     this.cameraState.moveCamera(delta);
   }
 
+  /**
+   * Moves home camera sideways of <code>delta</code>.
+   * @since 4.4 
+   */
+  public void moveCameraSideways(float delta) {
+    this.cameraState.moveCameraSideways(delta);
+  }
+
   /**
    * Elevates home camera of <code>delta</code>.
    */
@@ -332,6 +340,8 @@ public class HomeController3D implements Controller {
 
     public void moveCamera(float delta) {
     }
+    public void moveCameraSideways(float delta) {
+    }
 
     public void elevateCamera(float delta) {     
     }
@@ -779,6 +789,14 @@ public class HomeController3D implements Controller {
       home.setSelectedItems(Arrays.asList(new Selectable [] {this.observerCamera}));
     }
     
+    @Override
+    public void moveCameraSideways(float delta) {
+      this.observerCamera.setX(this.observerCamera.getX() - (float)Math.cos(this.observerCamera.getYaw()) * delta);
+      this.observerCamera.setY(this.observerCamera.getY() - (float)Math.sin(this.observerCamera.getYaw()) * delta);
+      // Select observer camera for user feedback
+      home.setSelectedItems(Arrays.asList(new Selectable [] {this.observerCamera}));
+    }
+    
     @Override
     public void elevateCamera(float delta) {
       float newElevation = this.observerCamera.getZ() + delta; 
