@@ -23,6 +23,8 @@ import java.util.List;
 import java.util.concurrent.Callable;
 
 import com.eteks.sweethome3d.model.Camera;
+import com.eteks.sweethome3d.model.Content;
+import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.InterruptedRecorderException;
 import com.eteks.sweethome3d.model.RecorderException;
 
@@ -37,7 +39,7 @@ public interface HomeView extends View {
   public enum ActionType {
       NEW_HOME, CLOSE, OPEN, DELETE_RECENT_HOMES, SAVE, SAVE_AS, SAVE_AND_COMPRESS,
       PAGE_SETUP, PRINT_PREVIEW, PRINT, PRINT_TO_PDF, PREFERENCES, EXIT, 
-      UNDO, REDO, CUT, COPY, PASTE, DELETE, SELECT_ALL,
+      UNDO, REDO, CUT, COPY, PASTE, DELETE, SELECT_ALL, SELECT_ALL_AT_ALL_LEVELS,
       ADD_HOME_FURNITURE, DELETE_HOME_FURNITURE, MODIFY_FURNITURE, 
       IMPORT_FURNITURE, IMPORT_FURNITURE_LIBRARY, IMPORT_TEXTURE, IMPORT_TEXTURES_LIBRARY,
       SORT_HOME_FURNITURE_BY_CATALOG_ID, SORT_HOME_FURNITURE_BY_NAME, 
@@ -57,7 +59,7 @@ public interface HomeView extends View {
       DISPLAY_HOME_FURNITURE_VALUE_ADDED_TAX, DISPLAY_HOME_FURNITURE_PRICE_VALUE_ADDED_TAX_INCLUDED,
       ALIGN_FURNITURE_ON_TOP, ALIGN_FURNITURE_ON_BOTTOM, ALIGN_FURNITURE_ON_LEFT, ALIGN_FURNITURE_ON_RIGHT, 
       ALIGN_FURNITURE_ON_FRONT_SIDE, ALIGN_FURNITURE_ON_BACK_SIDE, ALIGN_FURNITURE_ON_LEFT_SIDE, ALIGN_FURNITURE_ON_RIGHT_SIDE, ALIGN_FURNITURE_SIDE_BY_SIDE,
-      DISTRIBUTE_FURNITURE_HORIZONTALLY, DISTRIBUTE_FURNITURE_VERTICALLY,
+      DISTRIBUTE_FURNITURE_HORIZONTALLY, DISTRIBUTE_FURNITURE_VERTICALLY, RESET_FURNITURE_ELEVATION,
       GROUP_FURNITURE, UNGROUP_FURNITURE, EXPORT_TO_CSV, 
       SELECT, PAN, CREATE_WALLS, CREATE_ROOMS, CREATE_DIMENSION_LINES, CREATE_LABELS, DELETE_SELECTION,
       LOCK_BASE_PLAN, UNLOCK_BASE_PLAN, MODIFY_COMPASS, MODIFY_WALL, REVERSE_WALL_DIRECTION, SPLIT_WALL, MODIFY_ROOM, MODIFY_LABEL,
@@ -69,6 +71,7 @@ public interface HomeView extends View {
       DISPLAY_ALL_LEVELS, DISPLAY_SELECTED_LEVEL, MODIFY_3D_ATTRIBUTES, CREATE_PHOTO, CREATE_VIDEO, EXPORT_TO_OBJ,
       HELP, ABOUT}
   public enum SaveAnswer {SAVE, CANCEL, DO_NOT_SAVE}
+  public enum OpenDamagedHomeAnswer {REMOVE_DAMAGED_ITEMS, REPLACE_DAMAGED_ITEMS, DO_NOT_OPEN_HOME}
 
   /**
    * Enables or disables the action matching <code>actionType</code>.
@@ -104,6 +107,15 @@ public interface HomeView extends View {
    */
   public abstract String showOpenDialog();
 
+  /**
+   * Displays a dialog that lets user choose what he wants 
+   * to do with a damaged home he tries to open it. 
+   * @since 4.4 
+   */
+  public abstract OpenDamagedHomeAnswer confirmOpenDamagedHome(String homeName, 
+                                                               Home damagedHome, 
+                                                               List<Content> invalidContent);
+
   /**
    * Displays a content chooser open dialog to choose a language library.
    */
